from django.contrib import messages
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext as _

from .conf import settings


class DocumentsDefaultHookSet(object):

    def share_with_options(self, user, folder):
        """
        Return a list of users with whom `user` can share `folder`.
        """
        user_class = get_user_model()
        return user_class.objects.all()

    def can_share_folder(self, user, folder):
        """
        Return True if `user` can share `folder`.
        """
        return folder.parent_id is None and folder.author_id == user.id

    def storage_color(self, user_storage):
        """
        Return labels indicating amount of storage used.
        """
        p = user_storage.percentage
        if p >= 0 and p < 60:
            return "success"
        if p >= 60 and p < 90:
            return "warning"
        if p >= 90 and p <= 100:
            return "danger"
        raise ValueError("percentage out of range")

    def folder_created_message(self, request, folder):
        """
        Send messages.success message after successful folder creation.
        """
        messages.success(request, _("Folder {} was created".format(folder)))

    def document_created_message(self, request, document):
        """
        Send messages.success message after successful document creation.
        """
        messages.success(request, _("Document {} was created".format(document)))

    def folder_shared_message(self, request, user, folder):
        """
        Send messages.success message after successful share.
        """
        messages.success(request, _("Folder {} is now shared with {}".format(folder, user)))

    def folder_unshared_message(self, request, user, folder):
        """
        Send messages.success message after successful unshare.
        """
        messages.success(request, _("User {} no longer shares Folder {}".format(user, folder)))

    def document_deleted_message(self, request, document):
        """
        Send messages.success message after successful document deletion.
        """
        messages.success(request, _("Document has been deleted"))

    def folder_deleted_message(self, request, folder):
        """
        Send messages.success message after successful document deletion.
        """
        messages.success(request, _("Folder has been deleted"))

    def folder_pre_delete(self, request, folder):
        """
        Perform folder operations prior to deletions. For example, deleting all contents.
        """
        for m in folder.members():
            if m.__class__ == folder.__class__:
                self.folder_pre_delete(request, m)
            m.delete()


class HookProxy(object):

    def __getattr__(self, attr):
        return getattr(settings.DOCUMENTS_HOOKSET, attr)


hookset = HookProxy()



from django.core.files.uploadedfile import SimpleUploadedFile

from test_plus.test import TestCase as PlusTestCase

from ..exceptions import DuplicateFolderNameError, DuplicateDocumentNameError
from ..models import (
    Document,
    Folder
)


class BaseTest(PlusTestCase):

    def setUp(self):
        self.user = self.make_user("eldarion")


class FolderTestCase(BaseTest):

    def test_cannot_have_duplicate_sibling_folder_names(self):
        Folder.objects.create(name="Foo", author=self.user, modified_by=self.user)
        with self.assertRaises(DuplicateFolderNameError):
            Folder.objects.create(name="Foo", author=self.user, modified_by=self.user)

    def test_can_have_duplicate_folder_names_at_different_paths(self):
        foo = Folder.objects.create(name="Foo", author=self.user, modified_by=self.user)
        bar = Folder.objects.create(name="Bar", parent=foo, author=self.user, modified_by=self.user)
        Folder.objects.create(name="Foo", parent=bar, author=self.user, modified_by=self.user)
        with self.assertRaises(DuplicateFolderNameError):
            Folder.objects.create(name="Bar", parent=foo, author=self.user, modified_by=self.user)


class DocumentTestCase(BaseTest):

    def test_cannot_have_duplicate_sibling_document_names(self):
        simple_file = SimpleUploadedFile("delicious.txt", b"something tasty")
        Document.objects.create(name="Foo", author=self.user, file=simple_file, modified_by=self.user)
        with self.assertRaises(DuplicateDocumentNameError):
            Document.objects.create(name="Foo", author=self.user, file=simple_file, modified_by=self.user)

    def test_can_have_duplicate_document_names_at_different_paths(self):
        simple_file = SimpleUploadedFile("delicious.txt", b"something tasty")
        foo = Folder.objects.create(name="Foo", author=self.user, modified_by=self.user)
        bar = Folder.objects.create(name="Bar", parent=foo, author=self.user, modified_by=self.user)
        Document.objects.create(name="Foo", author=self.user, file=simple_file, modified_by=self.user)
        Document.objects.create(name="Bar", folder=foo, author=self.user, file=simple_file, modified_by=self.user)
        Document.objects.create(name="Foo", folder=bar, author=self.user, file=simple_file, modified_by=self.user)
        with self.assertRaises(DuplicateDocumentNameError):
            Document.objects.create(name="Bar", folder=foo, author=self.user, file=simple_file, modified_by=self.user)



from django.conf.urls import url

from . import views

urlpatterns = [
    url(r"^$", views.IndexView.as_view(),
        name="document_index"),
    url(r"^d/create/$", views.DocumentCreate.as_view(),
        name="document_create"),
    url(r"^d/(?P<pk>\d+)/$", views.DocumentDetail.as_view(),
        name="document_detail"),
    url(r"^d/(?P<pk>\d+)/download/$", views.DocumentDownload.as_view(),
        name="document_download"),
    url(r"^d/(?P<pk>\d+)/delete/$", views.DocumentDelete.as_view(),
        name="document_delete"),
    url(r"^f/create/$", views.FolderCreate.as_view(),
        name="folder_create"),
    url(r"^f/(?P<pk>\d+)/$", views.FolderDetail.as_view(),
        name="folder_detail"),
    url(r"^f/(?P<pk>\d+)/share/$", views.FolderShare.as_view(),
        name="folder_share"),
    url(r"^f/(?P<pk>\d+)/delete/$", views.FolderDelete.as_view(),
        name="folder_delete"),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings
import django.utils.timezone
import pinax.documents.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Document',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('name', models.CharField(max_length=255)),
                ('created', models.DateTimeField(default=django.utils.timezone.now)),
                ('modified', models.DateTimeField(default=django.utils.timezone.now)),
                ('file', models.FileField(upload_to=pinax.documents.models.uuid_filename)),
                ('author', models.ForeignKey(to=settings.AUTH_USER_MODEL, related_name='+')),
            ],
        ),
        migrations.CreateModel(
            name='DocumentSharedUser',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('document', models.ForeignKey(to='documents.Document')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Folder',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('name', models.CharField(max_length=140)),
                ('created', models.DateTimeField(default=django.utils.timezone.now)),
                ('modified', models.DateTimeField(default=django.utils.timezone.now)),
                ('author', models.ForeignKey(to=settings.AUTH_USER_MODEL, related_name='+')),
                ('modified_by', models.ForeignKey(to=settings.AUTH_USER_MODEL, related_name='+')),
                ('parent', models.ForeignKey(blank=True, null=True, to='documents.Folder')),
            ],
        ),
        migrations.CreateModel(
            name='FolderSharedUser',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('folder', models.ForeignKey(to='documents.Folder')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='UserStorage',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True, serialize=False)),
                ('bytes_used', models.BigIntegerField(default=0)),
                ('bytes_total', models.BigIntegerField(default=0)),
                ('user', models.OneToOneField(to=settings.AUTH_USER_MODEL, related_name='storage')),
            ],
        ),
        migrations.AddField(
            model_name='document',
            name='folder',
            field=models.ForeignKey(to='documents.Folder', null=True),
        ),
        migrations.AddField(
            model_name='document',
            name='modified_by',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL, related_name='+'),
        ),
        migrations.AlterUniqueTogether(
            name='foldershareduser',
            unique_together=set([('folder', 'user')]),
        ),
        migrations.AlterUniqueTogether(
            name='documentshareduser',
            unique_together=set([('document', 'user')]),
        ),
    ]



import mock

from django.core.files.uploadedfile import SimpleUploadedFile
from django.http import HttpResponse

from ..models import (
    Document,
    Folder,
)
from .test import BaseTest


class TestFolders(BaseTest):

    def setUp(self):
        super(TestFolders, self).setUp()
        self.create_urlname = "pinax_documents:folder_create"
        self.detail_urlname = "pinax_documents:folder_detail"
        self.share_urlname = "pinax_documents:folder_share"
        self.delete_urlname = "pinax_documents:folder_delete"

    def test_get_create_without_parent(self):
        """
        Ensure GET does not return a 'parent' folder in context.
        """
        with self.login(self.user):
            response = self.get(self.create_urlname)
            self.response_200(response)
            self.assertContext("parent", None)

    def test_get_create_with_parent(self):
        """
        Ensure GET returns a valid 'parent' folder in context.
        """
        parent_folder = Folder.objects.create(name="Parent", author=self.user)

        querystring_data = {"p": parent_folder.pk}
        with self.login(self.user):
            response = self.get(self.create_urlname, data=querystring_data)
            self.response_200(response)
            self.assertContext("parent", parent_folder)

    def test_get_create_with_illegal_parent(self):
        """
        Ensure GET fails if "p" querystring is invalid.
        """
        querystring_data = {"p": 555}
        with self.login(self.user):
            response = self.get(self.create_urlname, data=querystring_data)
            self.response_404(response)

    @mock.patch("django.contrib.messages.success")
    def test_post_create_without_parent(self, mock_messages):
        """
        Ensure POST creates folder without a parent.
        """
        folder_name = "Spindle"
        post_args = {"name": folder_name}
        with self.login(self.user):
            response = self.post(self.create_urlname, data=post_args, follow=True)
            self.response_200(response)
            created = self.get_context("object")
            self.assertFalse(created.parent)
            self.assertTrue(Folder.objects.get(name=folder_name))
            self.assertTrue(mock_messages.called)

    @mock.patch("django.contrib.messages.success")
    def test_post_create_with_parent(self, mock_messages):
        """
        Ensure POST creates folder with a parent.
        """
        parent_folder = Folder.objects.create(name="Parent", author=self.user)

        folder_name = "Spindle"
        post_args = {"name": folder_name, "parent": parent_folder.pk}
        with self.login(self.user):
            response = self.post(self.create_urlname, data=post_args, follow=True)
            self.response_200(response)
            created = self.get_context("object")
            self.assertEqual(created.parent, parent_folder)
            self.assertTrue(Folder.objects.get(name=folder_name))
            self.assertTrue(mock_messages.called)

    def test_post_create_with_illegal_parent(self):
        """
        Ensure POST does not creates folder with invalid parent.
        """
        folder_name = "Spindle"
        post_args = {"name": folder_name, "parent": 555}
        with self.login(self.user):
            response = self.post(self.create_urlname, data=post_args, follow=True)
            self.response_200(response)
            self.assertFalse("object" in self.last_response.context)
            self.assertFalse(Folder.objects.filter(name=folder_name))

    def test_post_create_with_duplicate_name(self):
        """
        Ensure POST does not create a folder with duplicate name
        """
        folder_name = "Spindle"
        Folder.objects.create(name=folder_name, author=self.user, modified_by=self.user)
        post_args = {"name": folder_name}
        with self.login(self.user):
            response = self.post(self.create_urlname, data=post_args, follow=True)
            self.response_200(response)
            self.assertTrue("{} already exists.".format(folder_name) in str(response.context["form"].errors))
            self.assertFalse("object" in self.last_response.context)

    def test_detail(self):
        """
        Ensure we can see folder detail.
        """
        parent_folder = Folder.objects.create(name="Parent", author=self.user)

        with self.login(self.user):
            self.get_check_200(self.detail_urlname, pk=parent_folder.pk)
            folder = self.get_context("object")
            self.assertEqual(folder, parent_folder)

    def test_duplicate_folder_name(self):
        """
        Do we allow Folders with the same name?
        """
        pass

    @mock.patch("django.contrib.messages.success")
    def test_delete(self, mock_messages):
        """
        Ensure we can delete a folder.
        """
        parent_folder = Folder.objects.create(name="Parent", author=self.user)

        with self.login(self.user):
            response = self.get(self.delete_urlname, pk=parent_folder.pk)
            self.response_200(response)
            self.assertTrue('pinax/documents/folder_confirm_delete.html' in response.template_name)

            response = self.post(self.delete_urlname, pk=parent_folder.pk, follow=True)
            self.response_200(response)
            self.assertFalse(Folder.objects.filter(pk=parent_folder.pk))
            self.assertTrue(mock_messages.called)

            response = self.get(self.detail_urlname, pk=parent_folder.pk)
            self.response_404(response)

    @mock.patch("django.contrib.messages.success")
    def test_delete_recursive(self, mock_messages):
        """
        Ensure that when we delete a folder with contents, those contents are also deleted.
        """
        parent_folder = Folder.objects.create(name="Parent", author=self.user)
        child_folder = Folder.objects.create(name="Child", author=self.user, parent=parent_folder)
        Folder.objects.create(name="Grand Child", author=self.user, parent=child_folder)

        child_file = SimpleUploadedFile("child.txt", b"Child document")
        child_document = Document.objects.create(name="Child Doc",
                                                 author=self.user,
                                                 file=child_file,
                                                 folder=parent_folder,
                                                 )
        grand_child_file = SimpleUploadedFile("grand_child.txt", b"Grand Child document")
        grand_child_document = Document.objects.create(name="Grand Child Doc",
                                                       author=self.user,
                                                       file=grand_child_file,
                                                       folder=child_folder,
                                                       )

        with self.login(self.user):
            response = self.post(self.delete_urlname, pk=parent_folder.pk, follow=True)
            self.response_200(response)
            self.assertFalse(Folder.objects.all())
            self.assertFalse(Document.objects.all())

            self.assertTrue(mock_messages.called)

            response = self.get("pinax_documents:document_detail", pk=child_document.pk)
            self.response_404(response)
            response = self.get("pinax_documents:document_detail", pk=grand_child_document.pk)
            self.response_404(response)

    def test_get_share(self):
        """
        Ensure sharing users are present in GET context.
        """
        sharing_user = self.make_user("sharing")
        folder = Folder.objects.create(name="Not Mine", author=self.user)
        folder.save()
        folder.share([sharing_user])
        # Add another user who is not sharing `folder`.
        self.make_user("nonsharing")

        with self.login(self.user):
            self.get(self.share_urlname, pk=folder.pk)
            self.assertInContext("participants")
            self.assertSequenceEqual(self.last_response.context["participants"], [sharing_user])

    @mock.patch("django.contrib.messages.success")
    def test_share_valid(self, mock_messages):
        """
        Ensure we can share a Folder with a valid user.
        """
        other_user = self.make_user("other")
        folder = Folder.objects.create(name="Mine", author=self.user)
        folder.save()

        post_args = {"participants": [other_user.pk]}
        with self.login(self.user):
            response = self.post(self.share_urlname, pk=folder.pk, data=post_args, follow=True)
            self.response_200(response)
            self.assertTrue(folder.shared)
            self.assertTrue(folder.shared_with(other_user))
            self.assertTrue(mock_messages.called)

    def test_share_non_author(self):
        """
        Ensure we cannot share a Folder we didn't author.
        """
        other_user = self.make_user("other")
        folder = Folder.objects.create(name="Not Mine", author=other_user)
        folder.save()
        folder.share([self.user])

        # `folder` is now associated with self.user, but he did not
        #  author `folder` and should not be able to share it.
        post_args = {"participants": [other_user]}
        with self.login(self.user):
            response = self.post(self.share_urlname, pk=folder.pk, data=post_args, follow=True)
            self.response_404(response)


class TestDocuments(BaseTest):

    def setUp(self):
        super(TestDocuments, self).setUp()
        self.create_urlname = "pinax_documents:document_create"
        self.detail_urlname = "pinax_documents:document_detail"
        self.download_urlname = "pinax_documents:document_download"

        self.file_contents = b"Golden Delicious apple"

    def test_get_create_without_folder(self):
        """
        Ensure GET does not return a 'folder' in context.
        """
        with self.login(self.user):
            response = self.get(self.create_urlname)
            self.response_200(response)
            self.assertContext("folder", None)

    def test_get_create_with_folder(self):
        """
        Ensure GET returns a valid 'folder' in context.
        """
        parent_folder = Folder.objects.create(name="Parent", author=self.user)

        querystring_data = {"f": parent_folder.pk}
        with self.login(self.user):
            response = self.get(self.create_urlname, data=querystring_data)
            self.response_200(response)
            self.assertContext("folder", parent_folder)

    def test_get_create_with_illegal_folder(self):
        """
        Ensure GET fails if "f" querystring is invalid.
        """
        querystring_data = {"f": 555}
        with self.login(self.user):
            response = self.get(self.create_urlname, data=querystring_data)
            self.response_404(response)

    @mock.patch("django.contrib.messages.success")
    def test_post_create_without_folder(self, mock_messages):
        """
        Ensure POST creates document without a folder.
        """
        simple_file = SimpleUploadedFile("delicious.txt", self.file_contents)

        post_args = {"name": "file", "file": simple_file}
        with self.login(self.user):
            response = self.post(self.create_urlname, data=post_args, follow=True)
            self.response_200(response)
            created = self.get_context("object")
            self.assertFalse(created.folder)
            self.assertTrue(Document.objects.get(name=simple_file.name))
            self.assertTrue(mock_messages.called)

    @mock.patch("django.contrib.messages.success")
    def test_post_create_with_parent(self, mock_messages):
        """
        Ensure POST creates document associated with a folder.
        """
        parent_folder = Folder.objects.create(name="Parent", author=self.user)
        simple_file = SimpleUploadedFile("delicious.txt", self.file_contents)

        post_args = {"name": "file", "file": simple_file, "folder": parent_folder.pk}
        with self.login(self.user):
            response = self.post(self.create_urlname, data=post_args, follow=True)
            self.response_200(response)
            created = self.get_context("object")
            self.assertEqual(created.folder, parent_folder)
            self.assertTrue(Document.objects.get(name=simple_file.name))
            self.assertTrue(mock_messages.called)

    def test_post_create_with_illegal_parent(self):
        """
        Ensure POST does not create a document with invalid folder.
        """
        simple_file = SimpleUploadedFile("delicious.txt", self.file_contents)

        post_args = {"name": "file", "file": simple_file, "folder": 555}
        with self.login(self.user):
            response = self.post(self.create_urlname, data=post_args, follow=True)
            self.response_200(response)
            self.assertFalse("object" in self.last_response.context)
            self.assertFalse(Document.objects.filter(name=simple_file.name))

    def test_post_create_with_duplicate_name(self):
        """
        Ensure POST does not create a document with duplicate name
        """
        simple_file = SimpleUploadedFile("delicious.txt", self.file_contents)
        Document.objects.create(name="delicious.txt", author=self.user, file=simple_file, modified_by=self.user)
        simple_file = SimpleUploadedFile("delicious.txt", self.file_contents)
        post_args = {"name": "file", "file": simple_file}
        with self.login(self.user):
            response = self.post(self.create_urlname, data=post_args, follow=True)
            self.response_200(response)
            self.assertTrue("delicious.txt already exists." in str(response.context["form"].errors))
            self.assertFalse("object" in self.last_response.context)

    def test_detail(self):
        """
        Ensure we can see document detail.
        """
        simple_file = SimpleUploadedFile("delicious.txt", self.file_contents)
        document = Document.objects.create(name="Honeycrisp",
                                           author=self.user,
                                           file=simple_file,
                                           )

        with self.login(self.user):
            self.get_check_200(self.detail_urlname, pk=document.pk)
            context_document = self.get_context("object")
            self.assertEqual(context_document, document)

    @mock.patch("django.contrib.messages.success")
    def test_valid_delete(self, mock_messages):
        """
        Ensure we can delete a valid Document.
        """
        simple_file = SimpleUploadedFile("delicious.txt", self.file_contents)
        document = Document.objects.create(name="Honeycrisp",
                                           author=self.user,
                                           file=simple_file,
                                           )

        doc_pk = document.pk
        with self.login(self.user):
            response = self.get(document.delete_url())
            self.response_200(response)
            self.assertTrue('pinax/documents/document_confirm_delete.html' in response.template_name)

            response = self.post("pinax_documents:document_delete", pk=doc_pk, follow=True)
            self.response_200(response)
            self.assertFalse(Document.objects.filter(pk=doc_pk))
            self.assertTrue(mock_messages.called)

            response = self.get(self.download_urlname, pk=document.pk)
            self.response_404(response)

    def test_invalid_delete(self):
        """
        Ensure we get error trying to delete an invalid Document.
        """
        doc_pk = 555
        with self.login(self.user):
            response = self.post("documents_document_delete", pk=doc_pk, follow=True)
            self.response_404(response)

    def test_download(self):
        """
        Ensure the requested Document file is served.
        """
        simple_file = SimpleUploadedFile("delicious.txt", self.file_contents)
        document = Document.objects.create(name="Honeycrisp",
                                           author=self.user,
                                           file=simple_file,
                                           )
        document.save()

        with self.login(self.user):
            # Verify `django.views.static.serve` is called to serve up the file.
            # See related note in .views.DocumentDownload.get().
            with mock.patch("django.views.static.serve") as serve:
                serve.return_value = HttpResponse()
                self.get_check_200(self.download_urlname, pk=document.pk)
                self.assertTrue(serve.called)



from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver

from .conf import settings
from .models import UserStorage, Document


@receiver(post_save, sender=settings.AUTH_USER_MODEL)
def ensure_userstorage(sender, **kwargs):
    if kwargs["created"]:
        user = kwargs["instance"]
        UserStorage.objects.create(user=user, bytes_total=(1024 * 1024 * 50))


# Receive the pre_delete signal and delete the file associated with the model instance.
@receiver(pre_delete, sender=Document)
def document_delete(sender, instance, **kwargs):
    # Pass false so FileField doesn't save the model.
    instance.file.delete(False)
